// Netify Agent Socket Sink Plugin
// Copyright (C) 2021-2023 eGloo Incorporated
// <http://www.egloo.ca>

#ifndef _NSP_PLUGIN_H
#define _NSP_PLUGIN_H

#include <nd-plugin.hpp>

#ifdef _ND_USE_WEBSOCKETS
#include <ixwebsocket/IXWebSocketServer.h>

using namespace ix;
#endif

#include "nsp-socket.hpp"

#define _NSP_DEFAULT_TCP_PORT 1750

class nspPlugin;

class nspChannel
{
public:
    nspChannel(const string &channel, const json &jconf,
      const string &default_port);
    virtual ~nspChannel() { CloseSockets(); }

    enum BindProtocol {
        BIND_UNKNOWN,
        BIND_UNIX,
        BIND_TCP,
        BIND_TCP_WS,
        BIND_TCP_WSS,
    };

    string channel;
    BindProtocol bind_proto;
    string bind_addr;
    string bind_port;

    void CreateSockets(void);
    void CloseSockets(void);

    void PollSockets(void);

    void QueuePayload(ndPluginSinkPayload *payload);

protected:
    friend class nspPlugin;

    nspSocketServer *server;
#ifdef _ND_USE_WEBSOCKETS
    string connection_id;
    WebSocketServer *server_ws;
    string tls_cert;
    string tls_key;
    string tls_ca;
    bool tls_verify_host;
#endif
    nspSocketClientMap clients;
    nspSocketBufferMap buffers;

    size_t QueueHeader(ndPluginSinkPayload *payload,
      nspSocketBufferMap::iterator &bi);

    void ClientAccept(void);
    void ClientHangup(nspSocketClientMap::iterator &ci);
};

class nspPlugin : public ndPluginSink
{
public:
    nspPlugin(const string &tag, const ndPlugin::Params &params);
    virtual ~nspPlugin();

    virtual void *Entry(void);

    virtual void DispatchEvent(ndPlugin::Event event,
      void *param = nullptr);

    virtual void GetVersion(string &version) {
        version = PACKAGE_VERSION;
    }

protected:
    atomic<bool> reload;

    typedef map<string, nspChannel> Channels;
    Channels channels;

    void Reload(void);
};

#endif  // _NSP_PLUGIN_H
